from subprocess import PIPE, Popen, check_output
from threading import Thread
from queue import Queue, Empty
from shlex import quote
from datetime import datetime

import os
import time
import re
import logging
import yaml
import py_cui

import recoverpy.errors as ERRORS
import recoverpy.window_handler as WINDOW_HANDLER
import recoverpy.logger as LOGGER
import recoverpy.saver as SAVER
import recoverpy.menu_with_block_display as BLOCK_DISPLAY_MENU
import recoverpy.search_menu as SEARCH_MENU
import recoverpy.block_menu as BLOCK_MENU

_LOGGER = logging.getLogger(__name__)


def parse_configuration():
    """Sets logging and saving paramters based on yaml conf file."""

    with open(os.path.join(os.path.dirname(__file__), "config.yaml")) as config_file:
        config = yaml.load(config_file, Loader=yaml.FullLoader)

    if config["save_directory"] == "":
        raise ERRORS.NoSavePath
    elif not os.path.isdir(config["save_directory"]):
        raise ERRORS.InvalidSavePath
    else:
        SAVER.set_save_path(config["save_directory"])

    if config["enable_logging"]:
        LOGGER.enable_logging()
    else:
        return

    if config["log_directory"] == "":
        LOGGER.disable_logging()
    elif not os.path.isdir(config["log_directory"]):
        raise ERRORS.InvalidLogPath
    else:
        LOGGER.set_log_file_path(config["log_directory"])


def main():
    parse_configuration()

    WINDOW_HANDLER.open_parameters_menu()



class NoSavePath(Exception):
    """Raised when save path is not set in config file"""

    def __init__(self):
        super().__init__(
            "No path entered for saving in config file. Verify your 'config.yaml' file."
        )


class InvalidSavePath(Exception):
    """Raised when save path is invalid"""

    def __init__(self):
        super().__init__(
            "Path entered for saving in config file is invalid. Verify your 'config.yaml' file."
        )


class InvalidLogPath(Exception):
    """Raised when log path is invalid"""

    def __init__(self):
        super().__init__(
            "Path entered for logging in config file is invalid. Verify your 'config.yaml' file.\
                \nYou can leave the location blank ( \"\" ) if you want to disable it."
        )


from datetime import datetime

import recoverpy.logger as LOGGER

_SAVE_PATH = None


def set_save_path(path: str):
    """Sets result save path based on config file"""

    global _SAVE_PATH

    _SAVE_PATH = path


def save_result(current_block: str, result: str):
    """Saves a single result in a text file"""

    global _SAVE_PATH

    file_name = "{save_location}{time}-{block}".format(
        save_location=_SAVE_PATH,
        time=datetime.now().strftime("recoverpy-save-%Y-%m-%d-%H%M%S"),
        block=current_block,
    )

    with open(file_name, "w") as save_file:
        save_file.write(result)

    LOGGER.write("info", "Output saved in file {file_name}".format(file_name=file_name))


def save_result_dict(results: dict):
    """Orders a results dictionnary by block numbers and the saves it in a text file."""

    global _SAVE_PATH

    ordered_blocks = sorted(results)

    final_output = ""

    for key in ordered_blocks:
        final_output += results[key]
        final_output += "\n"

    file_name = "{save_location}{time}".format(
        save_location=_SAVE_PATH,
        time=datetime.now().strftime("recoverpy-save-%Y-%m-%d-%H%M%S"),
    )

    with open(file_name, "w") as save_file:
        save_file.write(final_output)
    LOGGER.write("info", "Output saved in file {file_name}".format(file_name=file_name))



import py_cui

import recoverpy.parameters_menu as PARAMETERS_MENU
import recoverpy.search_menu as SEARCH_MENU
import recoverpy.block_menu as BLOCK_MENU
import recoverpy.logger as LOGGER

_PARAMETERS_MENU = None
_SEARCH_MENU = None
_BLOCK_MENU = None


def open_parameters_menu():
    """Starts a ParametersMenu instance."""
    global _PARAMETERS_MENU

    _PARAMETERS_MENU = py_cui.PyCUI(10, 10)
    _PARAMETERS_MENU.toggle_unicode_borders()
    _PARAMETERS_MENU.set_title("Retrieve deleted or overwritten text files")
    PARAMETERS_MENU.ParametersMenu(_PARAMETERS_MENU)
    _PARAMETERS_MENU.start()


def close_parameters_menu():
    """Stops the global ParametersMenu instance."""
    global _PARAMETERS_MENU

    _PARAMETERS_MENU.stop()


def open_search_menu(partition: str, string_to_search: str):
    """Starts a SearchMenu instance."""
    global _SEARCH_MENU

    _SEARCH_MENU = py_cui.PyCUI(10, 10)
    _SEARCH_MENU.toggle_unicode_borders()
    _SEARCH_MENU.set_title("View and explore found files")
    SEARCH_MENU.SearchMenu(
        _SEARCH_MENU, partition=partition, string_to_search=string_to_search
    )
    _SEARCH_MENU.start()


def close_search_menu():
    """Stops the global SearchMenu instance."""
    global _SEARCH_MENU

    _SEARCH_MENU.stop()


def open_block_menu(partition: str, block: str):
    global _BLOCK_MENU

    _BLOCK_MENU = py_cui.PyCUI(10, 10)
    _BLOCK_MENU.toggle_unicode_borders()
    _BLOCK_MENU.set_title("")
    BLOCK_MENU.BlockMenu(_BLOCK_MENU, partition=partition, initial_block=block)
    _BLOCK_MENU.start()



import py_cui

import recoverpy.menu_with_block_display as BLOCK_DISPLAY_MENU

import recoverpy.logger as LOGGER
import recoverpy.saver as SAVER


class BlockMenu(BLOCK_DISPLAY_MENU.MenuWithBlockDisplay):
    """Block menu is called in order to navigate more easily through partition blocks
    and eventually save multiple results in the same file.

    Args:
        BLOCK_DISPLAY_MENU (MenuWithBlockDisplay): Composition to inherit block display
        methods.

    Attributes:
        saved_blocks_dict (dict): Used to store block numbers associated with their text
            contents. Will be used to save it in a file.
        current_block (int): Partition block number currently displayed.
    """

    def __init__(self, master: py_cui.PyCUI, partition: str, initial_block: int):
        """Constructor for Block menu

        Args:
            master (py_cui.PyCUI): PyCUI constructor
            partition (str): System partition to search
            initial_block (int): Initial partition block number that will be displayed.
        """

        self.master = master

        self.partition = partition

        LOGGER.write("info", "Starting 'BlockMenu' CUI window")

        self.saved_blocks_dict = {}

        self.current_block = initial_block

        self.previous_button = self.master.add_button(
            "<",
            0,
            0,
            row_span=9,
            column_span=1,
            padx=1,
            pady=0,
            command=self.display_previous_block,
        )
        self.next_button = self.master.add_button(
            ">",
            0,
            9,
            row_span=9,
            column_span=1,
            padx=1,
            pady=0,
            command=self.display_next_block,
        )

        self.result_content_box = self.master.add_text_block(
            "Block content:", 0, 1, row_span=9, column_span=8, padx=1, pady=0
        )
        self.result_content_box.set_title(
            "Block {current_block}".format(current_block=str(self.current_block))
        )

        # Display initial block at opening
        self.display_block(self.current_block)

        self.add_result_button = self.master.add_button(
            "Add current block to file",
            9,
            0,
            row_span=1,
            column_span=5,
            padx=1,
            pady=0,
            command=self.add_block_to_file,
        )

        self.save_file_button = self.master.add_button(
            "Save file",
            9,
            5,
            row_span=1,
            column_span=3,
            padx=1,
            pady=0,
            command=self.save_file,
        )

        self.go_back_button = self.master.add_button(
            "Go back to previous menu",
            9,
            8,
            row_span=1,
            column_span=2,
            padx=1,
            pady=0,
            command=self.master.stop,
        )

    def add_block_to_file(self):
        """Stores currently displayed result in a dict to save it later."""

        if self.current_block in self.saved_blocks_dict:
            return

        self.master.show_message_popup("", "Result added to file")
        self.saved_blocks_dict[self.current_block] = self.current_result
        LOGGER.write(
            "debug",
            "Stored block {block} for future save".format(block=self.current_block),
        )

    def save_file(self):
        """Function called by user.
        Bundles results saving + popup message.
        """

        len_results = len(self.saved_blocks_dict)

        if len_results == 0:
            self.master.show_message_popup("", "No result to save yeet")
            return
        elif len_results == 1:
            self.master.show_message_popup("", "Result saved")
        else:
            self.master.show_message_popup(
                "", "{len_results} results saved".format(len_results=len_results)
            )

        SAVER.save_result_dict(self.saved_blocks_dict)



from setuptools import setup, find_packages

with open("requirements.txt", "r") as req_fp:
    required_packages = req_fp.readlines()

# Use README for long description
with open("README.md", "r") as readme_fp:
    long_description = readme_fp.read()

setup(
    name="recoverpy",
    version="0.0.1",
    author="PabloLec",
    author_email="pablolec@pm.me",
    description="A command line interface for recovering overwritten or deleted text data.",
    license="MIT License",
    keywords="data recovery cui",
    url="https://github.com/pabloleco/recoverpy",
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=find_packages(exclude=["tests", "docs"]),
    entry_points={
        "console_scripts": [
            "recoverpy = recoverpy:main",
        ],
    },
    install_requires=required_packages,
    package_data={"recoverpy": ["recoverpy/config.yaml"]},
    include_package_data=True,
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
    ],
)
